@@ -26,6 +26,7 @@
 #include <QScreen>
 
 #ifdef QT_DBUS_LIB
+#include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusMessage>
 #endif
@@ -103,6 +104,7 @@ void KRecentFilesActionPrivate::urlSelected(QAction *action)
     Q_EMIT q->urlSelected(url);
 }
 
+// TODO: remove this helper function, it will crash if you use it in a loop
 void KRecentFilesActionPrivate::removeAction(std::vector<RecentActionInfo>::iterator it)
 {
     Q_Q(KRecentFilesAction);
@@ -129,8 +131,9 @@ void KRecentFilesAction::setMaxItems(int maxItems)
         auto endIt = d->m_recentActions.begin() + difference;
         for (auto it = beginIt; it < endIt; ++it) {
             // Remove the action from the menus, action groups ...etc
-            d->removeAction(it);
+            delete KSelectAction::removeAction(it->action);
         }
+        d->m_recentActions.erase(beginIt, endIt);
     }
 }
 
@@ -210,7 +213,7 @@ void KRecentFilesAction::addUrl(const QUrl &url, const QString &name)
     static bool isKdeSession = qgetenv("XDG_CURRENT_DESKTOP") == "KDE";
     if (isKdeSession) {
         const QDBusConnection bus = QDBusConnection::sessionBus();
-        if (bus.isConnected()) {
+        if (bus.isConnected() && bus.interface()->isServiceRegistered(QStringLiteral("org.kde.ActivityManager"))) {
             const static QString activityService = QStringLiteral("org.kde.ActivityManager");
             const static QString activityResources = QStringLiteral("/ActivityManager/Resources");
             const static QString activityResouceInferface = QStringLiteral("org.kde.ActivityManager.Resources");
